/* ==========================================================================*/
/* - - - - - - - - - - - - -  a d c t r l . c  - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Implementation File for the adctrl.c module                            */
/*  Created   2011-12-02   msw                                               */
/*	Modified  2011-12-02   MW                                                */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* A/D control routines                                                      */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "iicctrl.h"
#include "timer.h"
#include "protocoldefs.h"
#include "adctrl.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
//#define ADTHRESH_XXX ( (1023*100)/(100+R) )	//formula with R in kOhms
#define ADTHRESH_160 ( (1023*100)/(100+15) )
#define ADTHRESH_80 ( (1023*100)/(100+22) )
#define ADTHRESH_40 ( (1023*100)/(100+33) )
#define ADTHRESH_30 ( (1023*100)/(100+47) )
#define ADTHRESH_20 ( (1023*100)/(100+68) )
#define ADTHRESH_17 ( (1023*100)/(100+100) )
#define ADTHRESH_15 ( (1023*100)/(100+150) )
#define ADTHRESH_12 ( (1023*100)/(100+220) )
#define ADTHRESH_10 ( (1023*100)/(100+330) )

#define ADTHRESH_DEF ( 0 )	//ID value of base board band

/*---------------------------------------------------------------------------*/
/*------------------------> M A C R O S <------------------------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static const uint32_t ID_TBL[BANDINDEX_DEF+1] =
{//index into this tabel is the BandIndex value so must match FREQBASE_TBL order
	ADTHRESH_160,
	ADTHRESH_80,
	ADTHRESH_40,
	ADTHRESH_30,
	ADTHRESH_20,
	ADTHRESH_17,
	ADTHRESH_15,
	ADTHRESH_12,
	ADTHRESH_10,
	ADTHRESH_DEF
};


/*---------------------------------------------------------------------------*/
/*------------------> L O C A L    P R O T O T Y P E S <---------------------*/
/*---------------------------------------------------------------------------*/


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | A D C t r l I n i t |                               */
/*                      ---------------------                                */
/*    Called to initialize the A/D control section.                          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ADCtrlInit( void )
{
	*AT91C_ADC_CR = AT91C_ADC_SWRST;
	*AT91C_ADC_MR = (5<<8)|AT91C_ADC_LOWRES_10_BIT | (0<<16) | (4<<24);
	//ADClock == MCK/(PRESCAL+1)*2 = 48MHz/(6*2) = 4 MHz
	//AD Startup Time == ((STARTUP+1)*8)/ADClock = 8/4MHz = 2uSec
	//AD SampleHold Time == SHTIM/MCLK = 4/4MHz = 1uSec
	ADRead(CH_ID);	//do dummy read
	Settings.BandIndex = GetBandIDIndex();	//read PA board ID to get band index
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------                                        */
/*                     | A D R e a d |                                       */
/*                      -------------                                        */
/*    Called to read the A/D channel.                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint16_t ADRead( uint32_t ch )
{
	*AT91C_ADC_CHER = ch;
	*AT91C_ADC_CR = AT91C_ADC_START;
	while( !(*AT91C_ADC_SR & ch)  ){};
	*AT91C_ADC_CHDR = ch;
	return (*AT91C_ADC_LCDR) & 0x3FF;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -----------------------------                           */
/*                  | G e t B a n d I D I n d e x |                          */
/*                   -----------------------------                           */
/*    Returns BandIndex based on resistor value on current plugin PA board.  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint32_t GetBandIDIndex(void)
{
uint32_t i;
int adval;
int min = 1000000;	//init to big number
int dif;
uint32_t minindex = 0;
	//Search for nearest AD value to table value
	adval = (int)ADRead(CH_ID);
	for(i=0; i<=BANDINDEX_DEF; i++)
	{
		dif = adval - ID_TBL[i];
		if(dif<0)
			dif = -dif;
		if(dif < min)
		{	//keep track of minimum difference and index between AD val and table value
			min = dif;
			minindex = i;
		}
	}
	return minindex;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                         ---------------                                   */
/*                        | G e t V S W R |                                  */
/*                         ---------------                                   */
/*    Returns VSWR x 100.  Also sets yellow LED Intensity.                   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int GetVSWR(void)
{
int vfwd = (int)ADRead(CH_FWD);
int vrev = (int)ADRead(CH_REV);
int gamma;
	if( (vfwd<5) && (vrev<5))
		return 0;
	if( CI_OPMODE_TUNE == Settings.OpMode)
		SetYLedPWM(vrev*10);
	vrev *= 1000;
	if(0==vfwd)
		vfwd = 1;
	gamma = vrev/vfwd;
//printf("f=%d r=%d g=%d",vfwd,vrev,gamma);
	if(gamma>980)	//clamp swr to 99.00
		gamma = 980;
	return (100*(1000+gamma)/(1000-gamma));
}
