/* ==========================================================================*/
/* - - - - - - - - - - - - -  i i c c t r l . c  - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Implementation File for the iicctrl.c module                           */
/*  Created   2010-08-01	msw                                              */
/*  Modified  2010-08-01    msw                                              */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* IIC control routines                                                      */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "iicctrl.h"
#include "protocoldefs.h"
#include "timer.h"
#include "string.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define BASE_160 1838000	//base frequencies for the ham bands
#define BASE_80 3594000
#define BASE_40 7040000
#define BASE_30 10140100
#define BASE_20 14097000
#define BASE_17 18106000
#define BASE_15 21096000
#define BASE_12 24926000
#define BASE_10 28126000
#define BASE_DEF 7040000

/*---------------------------------------------------------------------------*/
/*------------------------> M A C R O S <------------------------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
tSettings Settings;
BOOL NeedToSaveSettings;
static xSemaphoreHandle xIICSemaphore = NULL;

const uint32_t FREQBASE_TBL[BANDINDEX_DEF+1] =
{
	BASE_160,
	BASE_80,
	BASE_40,
	BASE_30,
	BASE_20,
	BASE_17,
	BASE_15,
	BASE_12,
	BASE_10,
	BASE_DEF
};

/*---------------------------------------------------------------------------*/
/*------------------> L O C A L    P R O T O T Y P E S <---------------------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | I I C C t r l I n i t |                             */
/*                      -----------------------                              */
/*    Called to initialize the IIC devices and IIC port.                     */
/*  Timer 2 must be initialized first.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void IICCtrlInit( void )
{
	//* Configure PIO controllers to periph mode for TWI (I2C)
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) SDA    ) |
 		((unsigned int) SCL   ),// Peripheral A
 		0); // Peripheral B
	//Set up I2C Port.
	AT91F_PMC_EnablePeriphClock( AT91C_BASE_PMC, 1 << AT91C_ID_TWI);
	*AT91C_TWI_CR = AT91C_TWI_MSEN;
	*AT91C_TWI_CWGR = (58<<8) + 58;	// CHDIV = MCLK/(400000*2) - 3
	xIICSemaphore = xSemaphoreCreateMutex();
	GetSettings();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | G e t S e t t i n g s |                             */
/*                      -----------------------                              */
/*    Retrieve program settings from EEROM                                   */
/*  Perform limit check to make sure all settings are in range               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void GetSettings()
{
BOOL SetDefaults = FALSE;
int i;
	//Load settings from EEROM
	for(i=0; i<10; i++)
		Settings.FreqOffset[i] = Read32FRAM(EE_FREQOFFSET + (4*i) );
	Settings.WsprInterval =	ReadFRAM(EE_WSPRINTERVAL);
	Settings.WsprPower = ReadFRAM(EE_WSPRPOWER);

	for(i=0; i<10; i++)
		Settings.CallSign[i] = ReadFRAM(EE_CALLSIGN + i);
	//limit checks 
	if( Settings.OpMode > CI_OPMODE_BOOT  )
		SetDefaults = TRUE;
	else if((Settings.FreqOffset[0]>FREQ_MAX) ||
			(Settings.FreqOffset[1]>FREQ_MAX) ||
			(Settings.FreqOffset[2]>FREQ_MAX) ||
			(Settings.FreqOffset[3]>FREQ_MAX) ||
			(Settings.FreqOffset[4]>FREQ_MAX) ||
			(Settings.FreqOffset[5]>FREQ_MAX) ||
			(Settings.FreqOffset[6]>FREQ_MAX) ||
			(Settings.FreqOffset[7]>FREQ_MAX) ||
			(Settings.FreqOffset[8]>FREQ_MAX) )
		SetDefaults = TRUE;
	else if( Settings.FreqOffset[0]>FREQ_MAX  )
		SetDefaults = TRUE;
	else if( Settings.BandIndex>BANDINDEX_10   )
		Settings.BandIndex = 0;
	else if( (Settings.WsprInterval>INTERVAL_MAX ) || (Settings.WsprInterval<INTERVAL_MIN )  )
		SetDefaults = TRUE;
	else if( (Settings.WsprPower>POWER_MAX ) || (Settings.WsprPower<POWER_MIN )  )
		SetDefaults = TRUE;
	else if( (0==Settings.CallSign[0]) || (strlen( Settings.CallSign) > 8) )
		SetDefaults = TRUE;

	if(SetDefaults)
		SetFactoryDefaults();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*              -------------------------------------                        */
/*             | S e t F a c t o r y D e f a u l t s |                       */
/*              -------------------------------------                        */
/*    Set to factory default Settings                                        */
/*  Set	NeedToSaveSettings to save modified to EEROM                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetFactoryDefaults()
{
int i;
	for(i=0; i<10; i++)
		Settings.FreqOffset[i] = 100;
	Settings.WsprInterval = INTERVAL_DEF;
	Settings.WsprPower = POWER_DEF;
	Settings.CallSign[0]='N'; Settings.CallSign[1]='o';
	Settings.CallSign[2]=' '; Settings.CallSign[3]='C';
	Settings.CallSign[4]='a'; Settings.CallSign[5]='l';
	Settings.CallSign[6]='l'; Settings.CallSign[7]=0;
	Settings.CallSign[8]=0; Settings.CallSign[9]=0;
	NeedToSaveSettings = TRUE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | S a v e A l l |                                    */
/*                       ---------------                                     */
/*    Save all settings to eerom                                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SaveAll()
{
int i;
	for(i=0; i<10; i++)
		Write32FRAM(EE_FREQOFFSET + (4*i), Settings.FreqOffset[i]);
	WriteFRAM(EE_WSPRINTERVAL, Settings.WsprInterval);
	WriteFRAM(EE_WSPRPOWER, Settings.WsprPower);
	for(i=0; i<=strlen(Settings.CallSign); i++)
		WriteFRAM(EE_CALLSIGN+i, Settings.CallSign[i]);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------                                      */
/*                     | R e a d I I C |                                     */
/*                      ---------------                                      */
/*    Called to Read data from an I2C device register.                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint8_t ReadIIC( uint8_t adr, uint8_t Reg )
{
uint8_t ret = 0;
	if(xSemaphoreTake( xIICSemaphore, ( portTickType ) 10 ) == pdTRUE )
	{
		ret = *AT91C_TWI_RHR;		//dummy read according to SAM7 errata
		*AT91C_TWI_IADR = (uint32_t)Reg;
		*AT91C_TWI_MMR = ((uint32_t)adr<<16)|AT91C_TWI_IADRSZ_1_BYTE|AT91C_TWI_MREAD;
		*AT91C_TWI_CR = AT91C_TWI_STOP|AT91C_TWI_START;	//START/STOP is automatic?
		Delayus(30);
		while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
		ret = (uint8_t)*AT91C_TWI_RHR;
		xSemaphoreGive( xIICSemaphore );
	}
	return ret;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | W r i t e I I C |                                   */
/*                      -----------------                                    */
/*    Called to Send data to an I2C device register.                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteIIC( uint8_t adr, uint8_t Reg, uint8_t Data )
{
uint32_t x __attribute__((unused));
	if(xSemaphoreTake( xIICSemaphore, ( portTickType ) 10 ) == pdTRUE )
	{
		x = *AT91C_TWI_RHR;		//dummy read according to SAM7 errata
		*AT91C_TWI_MMR = ((uint32_t)adr<<16)|AT91C_TWI_IADRSZ_1_BYTE;
		*AT91C_TWI_IADR = (uint32_t)Reg;
		*AT91C_TWI_THR = Data;
		*AT91C_TWI_CR = AT91C_TWI_STOP|AT91C_TWI_START;	//START/STOP is automatic?
		Delayus(30);
		while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
	}
	xSemaphoreGive( xIICSemaphore );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------                                      */
/*                     | W r i t e S i |                                     */
/*                      ---------------                                      */
/*    Called to Write a Byte to the Si5338 at Adr.                           */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteSi( uint8_t Adr, uint8_t Data )
{
uint32_t x __attribute__((unused));
	if(xSemaphoreTake( xIICSemaphore, ( portTickType ) 10 ) == pdTRUE )
	{
		x = *AT91C_TWI_RHR;		//dummy read according to SAM7 errata
		*AT91C_TWI_MMR = (SI5338_ADR<<16)|AT91C_TWI_IADRSZ_1_BYTE;
		*AT91C_TWI_IADR = Adr;
		*AT91C_TWI_THR = Data;
		*AT91C_TWI_CR = AT91C_TWI_START|AT91C_TWI_MSEN|AT91C_TWI_STOP;
		Delayus(30);
		while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
	}
	xSemaphoreGive( xIICSemaphore );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------                                        */
/*                     | R e a d S i |                                       */
/*                      -------------                                        */
/*    Called to Read a Byte from the Si5338 at Adr.                          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint8_t ReadSi( uint8_t Adr )
{
uint8_t ret = 0;
	if(xSemaphoreTake( xIICSemaphore, ( portTickType ) 10 ) == pdTRUE )
	{
		*AT91C_TWI_IADR = Adr;
		*AT91C_TWI_MMR = (SI5338_ADR<<16)|AT91C_TWI_IADRSZ_1_BYTE|AT91C_TWI_MREAD;
		*AT91C_TWI_CR = AT91C_TWI_START|AT91C_TWI_MSEN|AT91C_TWI_STOP;
		Delayus(30);
		while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
		ret = (uint8_t)*AT91C_TWI_RHR;
		xSemaphoreGive( xIICSemaphore );
	}
	return ret;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | W r i t e F R A M |                                 */
/*                      -------------------                                  */
/*    Called to Write a Byte to the FRAM at Adr.                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteFRAM( uint16_t Adr, uint8_t Data )
{
	if(xSemaphoreTake( xIICSemaphore, ( portTickType ) 10 ) == pdTRUE )
	{
		if( Data != ReadFRAM( Adr ) )
		{
			*AT91C_TWI_IADR = Adr;
#ifdef USE_64KFRAM
			*AT91C_TWI_MMR = (FRAM_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_2_BYTE;
#else
			*AT91C_TWI_MMR = (FRAM_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_1_BYTE;
#endif
			*AT91C_TWI_CR = AT91C_TWI_START|AT91C_TWI_MSEN|AT91C_TWI_STOP;
			*AT91C_TWI_THR = Data;
			Delayus(30);
			while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
#ifdef USE_4KEEROM
			vTaskDelay(5);
#endif
		}
	}
	xSemaphoreGive( xIICSemaphore );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | R e a d F R A M |                                   */
/*                      -----------------                                    */
/* Called to Read a Byte from the FRAM at Adr. (only call before RTOS is run)*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint8_t ReadFRAM( uint16_t Adr )
{
uint8_t ret = 0;
	*AT91C_TWI_IADR = Adr;
#ifdef USE_64KFRAM
	*AT91C_TWI_MMR = (FRAM_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_2_BYTE|AT91C_TWI_MREAD;
#else
	*AT91C_TWI_MMR = (FRAM_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_1_BYTE|AT91C_TWI_MREAD;
#endif
	*AT91C_TWI_CR = AT91C_TWI_START|AT91C_TWI_MSEN|AT91C_TWI_STOP;
	Delayus(30);
	while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
	ret = (uint8_t)*AT91C_TWI_RHR;
	return ret;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | W r i t e 3 2 F R A M |                             */
/*                      -----------------------                              */
/*    Called to Write a 32bit word to the FRAM at byte Adr.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Write32FRAM( uint16_t Adr, uint32_t Data )
{
t32BITS tmp;
	tmp.all = Data;
	WriteFRAM( Adr, tmp.bytes.b0);
	WriteFRAM( Adr+1, tmp.bytes.b1);
	WriteFRAM( Adr+2, tmp.bytes.b2);
	WriteFRAM( Adr+3, tmp.bytes.b3);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | W r i t e 1 6 F R A M |                             */
/*                      -----------------------                              */
/*    Called to Write a 16bit word to the FRAM at byte Adr.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Write16FRAM( uint16_t Adr, uint16_t Data )
{
t16BITS tmp;
	tmp.all = Data;
	WriteFRAM( Adr, tmp.bytes.b0);
	WriteFRAM( Adr+1, tmp.bytes.b1);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | R e a d 1 6 F R A M |                               */
/*                      ---------------------                                */
/*    Called to Read a 16 bit word from the FRAM at byte Adr.                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint16_t Read16FRAM( uint16_t Adr )
{
t16BITS tmp;
	tmp.bytes.b0 = ReadFRAM( Adr);
	tmp.bytes.b1 = ReadFRAM( Adr+1);
	return tmp.all;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | R e a d 3 2 F R A M |                               */
/*                      ---------------------                                */
/*    Called to Read a 32 bit word from the FRAM at byte Adr.                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
uint32_t Read32FRAM( uint16_t Adr )
{
t32BITS tmp;
	tmp.bytes.b0 = ReadFRAM( Adr);
	tmp.bytes.b1 = ReadFRAM( Adr+1);
	tmp.bytes.b2 = ReadFRAM( Adr+2);
	tmp.bytes.b3 = ReadFRAM( Adr+3);
	return tmp.all;
}


