/* ==========================================================================*/
/* - - - - - - - - - - - -   T e r m . c   - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Implementation File for the uart terminal module                       */
/*	Created   2011-09-21   M. Wheatley                                       */
/*	Modified  2011-09-21   MW                                                */
/* WSPR Beacon   AT91SAM7256 Terminal uart handler                           */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "term.h"
#include "irq.h"
#include "iicctrl.h"
#include "keyparser.h"
#include "protocoldefs.h"
#include "wspr.h"
#include "gps.h"
#include "adctrl.h"
#include "si5338.h"
#include "wsprencoder.h"
#include "adctrl.h"
#include "timer.h"
 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
//* Standard Asynchronous Mode : 8 bits , 1 stop , no parity
#define TERMASYNC_MODE ( AT91C_US_USMODE_NORMAL + \
                        AT91C_US_NBSTOP_1_BIT + \
                        AT91C_US_PAR_NONE + \
                        AT91C_US_CHRL_8_BITS + \
                        AT91C_US_CLKS_CLOCK )

#define TERM_BAUD 115200

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
uint8_t TermRxQueue[TERMQSIZE];
uint8_t TermTxQueue[TERMQSIZE];
volatile uint32_t TermRxHead;
static uint32_t TermRxTail;
volatile uint32_t TermTxTail;
uint32_t TermTxHead;
xSemaphoreHandle xTermRxSemaphore = NULL;
static uint32_t PresSw;
static uint32_t LastSw;
static uint8_t LastMode;
static uint8_t StatusTimer;
static BOOL LastTimeStatus;
static uint32_t LastTotal;

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void TermTask( void *pvParmeters );
static void ProcessOpMode();
static void PrintStatus(BOOL Init);
static void ClearLine();

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        ------------------                                 */
/*                       |  T e r m I n i t |                                */
/*                        ------------------                                 */
/*    Called to initialize the Terminal debug uart port.                     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void TermInit( )
{
	TermRxHead = 0;
	TermRxTail = 0;
	TermTxHead = 0;
	TermTxTail = 0;
#if REV1_PCB
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_SYS ) ;//enable debug unit clock
	AT91F_PIO_CfgPeriph(
		AT91C_BASE_PIOA, // PIO controller base address
		((unsigned int) DTXD    ) |
		((unsigned int) DRXD    ), // Peripheral A
		0); // Peripheral B
	AT91C_BASE_DBGU->DBGU_IDR = 0xFFFFFFFF;
	AT91C_BASE_DBGU->DBGU_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;
	AT91C_BASE_DBGU->DBGU_CR = AT91C_US_RSTSTA;
	AT91C_BASE_DBGU->DBGU_BRGR = ((MCK*10)/(16*TERM_BAUD) + 5) /10;	//use nearest integer;
	AT91C_BASE_DBGU->DBGU_MR = TERMASYNC_MODE;
	// Enable usart
	AT91C_BASE_DBGU->DBGU_CR = AT91C_US_RXEN | AT91C_US_TXEN;
 //* Config Debug Usart interrupt(is hooked into System interrupt handler (PortISR.c)
	*AT91C_DBGU_IER = AT91C_US_RXRDY;
#else
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US0 ) ;
	//* Configure uart PIO controllers to periph mode
	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
		((unsigned int) RXD0    ) |
		((unsigned int) TXD0    ),// Peripheral A
		0); // Peripheral B
	AT91C_BASE_US0->US_IDR = 0xFFFFFFFF;
	AT91C_BASE_US0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;
	AT91C_BASE_US0->US_CR = AT91C_US_RSTSTA;
	AT91C_BASE_US0->US_BRGR = ((MCK*10)/(16*TERM_BAUD) + 5) /10;	//use nearest integer;
	AT91C_BASE_US0->US_MR = TERMASYNC_MODE;
	// Enable usart
	AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;
	//* Config Usart interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_US0, USART0_INTERRUPT_LEVEL,
			AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Usart0IrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US0);
	*AT91C_US0_IER = AT91C_US_RXRDY;
#endif
	xTaskCreate( TermTask, ( const signed portCHAR * const ) "TERM",
				configMINIMAL_STACK_SIZE, NULL, TERM_TASK_PRIORITY, ( xTaskHandle * ) NULL );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | T e r m T a s k |                                   */
/*                      -----------------                                    */
/*    Task that process Terminal usart rx bytes                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void TermTask( void *pvParmeters )
{
uint8_t ch;
	PresSw = (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & (SW1|SW2|SW4|SW8)) >> 15;
	LastMode = 255;
	LastTimeStatus = FALSE;
	LastTotal = 0;
	StatusTimer = 0;
	PresSw = (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & (SW1|SW2|SW4|SW8)) >> 11;
	LastSw = PresSw & 0x0F;	//force switch read
	vSemaphoreCreateBinary( xTermRxSemaphore );
	xSemaphoreTake( xTermRxSemaphore, ( portTickType )10 );
	vTaskDelay(100);
	printf("\r\n-- WSPR Beacon --\r\r\n\n");
	printf("Compiled: %s  %s\r\n", __DATE__, __TIME__);
	Settings.BandIndex = GetBandIDIndex();	//read PA board ID to get band index
	for( ;; )	//loop forever in this task
    {
		if( xSemaphoreTake( xTermRxSemaphore, ( portTickType )100 ) == pdTRUE )
		{
			while( TermRxHead != TermRxTail)
			{
				ch = TermRxQueue[TermRxTail++];
				if(TermRxTail >= TERMQSIZE)
					TermRxTail = 0;
				if(	CI_OPMODE_SETUP == Settings.OpMode)	//process keys if in Off mode
					ProcessNewByte (ch );
				if(	('\r' == ch) && (CI_OPMODE_WSPR == Settings.OpMode))	//process CR if in WSPR mode
					PrintStatus(TRUE);
			}
		}
		else	//nothing coming in from Terminal serial port
		{
			ProcessOpMode();	//process any operational mode that is active at 100mS rate
			PresSw = (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & (SW1|SW2|SW4|SW8)) >> 11;
			PresSw = ~PresSw & 0x0F;
			if( LastSw != PresSw)
			{
//				printf("SW = %u\r\n",PresSw);
				LastSw = PresSw;
				switch(PresSw)
				{
					case 0:			//force bootloader on pwr up
						Settings.OpMode = CI_OPMODE_BOOT;
						break;
					case 1:
					case 2:
					case 3:
					case 4:
					case 5:
					case 6:
					case 7:
					case 8:
					case 9:
					case 10:
					case 11:
						Settings.OpMode = CI_OPMODE_WSPR;
						break;
					case 12:
						Settings.OpMode = CI_OPMODE_GPSRAW;
						break;
					case 13:
						Settings.OpMode = CI_OPMODE_TUNE;
						break;
					case 14:
						Settings.OpMode = CI_OPMODE_SETUP;
						break;
					case 15:
						Settings.OpMode = CI_OPMODE_GPS;
						break;
				}//end switch
			}//end if
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | P r o c e s s O p M o d e |                         */
/*                      ---------------------------                          */
/* Called to process active modes                                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ProcessOpMode()
{
int changed;
int vswr;
	changed = (LastMode != Settings.OpMode);
	switch(	Settings.OpMode)
	{
		case CI_OPMODE_SETUP:
			RfOff();
			if(changed)
			{
				printf("\r\nTx Off Terminal Setup Mode");
				KeyParserInit();
			}
			break;
		case CI_OPMODE_BOOT:
			RfOff();
			if(changed)
			{
				printf("\r\nPower Cycle to Enter Boot Mode");
			}
			break;
		case CI_OPMODE_TUNE:
			if(changed)
			{
				RfOn();
				printf("\r\nTune Mode\r\n");
			}
			//manage swr measuring
			printf("\r           \r");
			vswr = GetVSWR();
			SetRLedPWM(vswr-100);
			printf("VSWR = %d.%d%d ", vswr/100, (vswr%100)/10, vswr%10);
			break;
		case CI_OPMODE_GPS:
			RfOff();
			break;
		case CI_OPMODE_GPSRAW:
			RfOff();
			break;
		case CI_OPMODE_WSPR:
			if(changed)
				RfOff();
			if(0==StatusTimer-- || changed)
			{
				if(changed)
					printf("\r\r\n\n");
				StatusTimer = 9;	//update time in 100's milliseconds
				if(!TimeIsAccurate)
				{
					ClearLine();
					printf("No Lock -- GPS Sats=%d",NumSats);
					LastTimeStatus = FALSE;
				}
				else
				{
					PrintStatus(changed || !LastTimeStatus);
					LastTimeStatus = TRUE;
				}
			}
			break;
	}
	if(changed)
		LastMode = Settings.OpMode;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | P r i n t S t a t u s |                             */
/*                      -----------------------                              */
/* Called to output status info to uart while in WSPR mode.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void PrintStatus(BOOL Init)
{
char FreqStrBuf[16];
	if(Init)
	{
		printf("\r\r\n\nCall = %s  Grid Square = %s",Settings.CallSign ,MHStr);
		printf("\r\nLatitude = %s  Longitude = %s",LatStr, LonStr);
		GetFrequencyStr(FREQBASE_TBL[Settings.BandIndex] + Settings.FreqOffset[Settings.BandIndex], FreqStrBuf);
		printf("\r\nFreq = %s MHz", FreqStrBuf);
		printf("\r\nInterval = %d Min\r\n", (int)Settings.WsprInterval);
	}
	if(Running)
	{
		ClearLine();
		printf("!Transmitter On! UTC = %d:%d:%d  Symb = %d  ", HourTimer, MinuteTimer, SecondTimer, (int)(162-SymbCount));
	}
	else
	{
		if(LastTotal != TotalTransmissions)
		{
			printf("\n");
			LastTotal = TotalTransmissions;
		}
		ClearLine();
		printf("Sats = %d   UTC = %d:%d:%d  Next = %d:%d Total = %d  ", NumSats, HourTimer, MinuteTimer, SecondTimer, MinuteCounter, 59-SecondTimer, (int)TotalTransmissions);
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | C l e a r L i n e |                                 */
/*                      -------------------                                  */
/* Called to clear current line.                                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ClearLine()
{
	printf("\r                                                      \r");
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | S e n d T e r m S t r |                             */
/*                      -----------------------                              */
/* Called to send string to the usart.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendTermStr( char* pStr)
{
#if REV1_PCB
	*AT91C_DBGU_IDR = AT91C_US_TXRDY;
	while(0 != *pStr)
	{
		TermTxQueue[TermTxHead++] = *pStr++;
		TermTxHead &= (TERMQSIZE-1);
	}
	*AT91C_DBGU_IER = AT91C_US_TXRDY;
#else
	*AT91C_US0_IDR = AT91C_US_TXRDY;
	while(0 != *pStr)
	{
		TermTxQueue[TermTxHead++] = *pStr++;
		TermTxHead &= (TERMQSIZE-1);
	}
	*AT91C_US0_IER = AT91C_US_TXRDY;
#endif
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------------                            */
/*                     | S e n d T e r m U a r t |                           */
/*                      -------------------------                            */
/* Called to send one byte out the usart.                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendTermUart(unsigned char ch)
{
#if REV1_PCB
	*AT91C_DBGU_IDR = AT91C_US_TXRDY;
	TermTxQueue[TermTxHead++] = ch;
	TermTxHead &= (TERMQSIZE-1);
	*AT91C_DBGU_IER = AT91C_US_TXRDY;
#else
	*AT91C_US0_IDR = AT91C_US_TXRDY;
	TermTxQueue[TermTxHead++] = ch;
	TermTxHead &= (TERMQSIZE-1);
	*AT91C_US0_IER = AT91C_US_TXRDY;
#endif
}
