/* ==========================================================================*/
/* - - - - - - - - - - - - -  T i m e r . c  - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2010-05-29   M. Wheatley                                       */
/*	Modified  2010-05-29   MW                                                */
/*.........................................................................  */
/* WSPR Beacon   AT91SAM7256 Timer management                                */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "AT91SAM7S256.h"
#include "common.h"
#include "timer.h"
#include "irq.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
volatile int32_t CaptureTime;
volatile int32_t LastCaptureTime;
volatile int32_t PpsPeriod;
volatile int32_t PpsTimeout;

volatile uint8_t SecondTimer;
volatile uint8_t MinuteTimer;
volatile uint8_t HourTimer;
volatile uint8_t Timer100Hz;



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | T i m e r I n i t |                                 */
/*                      -------------------                                  */
/*    Called to initialize the Timers.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void TimerInit ( void )
{
int dummy __attribute__((unused));
	//* Open timer0	(WSPR bit timer)
	//* First, enable the clock of the TIMER
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC0 ) ;
	AT91C_BASE_TCB->TCB_BMR = AT91C_TCB_TC0XC0S_NONE |AT91C_TCB_TC1XC1S_NONE|AT91C_TCB_TC2XC2S_NONE;

	//* Disable the clock and the interrupts
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC0->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit
	dummy = AT91C_BASE_TC0->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV5_CLOCK|AT91C_TC_WAVESEL_UP_AUTO|
								AT91C_TC_WAVE; //MCK/(1024)  46.875KHz clk to timer0
	//* Enable the clock
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN;
	//* Config Timer 0 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC0, TIMER0_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Timer0IrqService);
	AT91C_BASE_TC0->TC_IER = AT91C_TC_CPCS;  //  IRQ enable CPC
	AT91C_BASE_TC0->TC_RC = 32000;		//1.46Hz bit rate

//-------------------------------------------------
	//* Open timer1	(1PPS capture timer)
	//* First, enable the clock of the TIMER
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC1 ) ;
	//* Disable the clock and the interrupts
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC1->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit	dummy = AT91C_BASE_TC1->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC1->TC_CMR = AT91C_TC_CLKS_TIMER_DIV1_CLOCK |
			AT91C_TC_ETRGEDG_NONE |	// no external reset (trigger) on falling
			AT91C_TC_LDRA_RISING |	// Load A on rising
			AT91C_TC_LDRB_RISING; // Load B on rising
  //* Config Timer 1 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC1, TIMER1_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Timer1IrqService);
	AT91C_BASE_TC1->TC_IER = AT91C_TC_COVFS | AT91C_TC_LDRAS | AT91C_TC_LDRBS;
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC1);
	//* Enable the clock and start counter free running
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKEN | AT91C_TC_SWTRG;
	CaptureTime = 0;
	LastCaptureTime = 0;
	PpsTimeout = 0;
//-------------------------------------------------
	//* Open timer2	(Internal TOD timer)
	//* First, enable the clock of TIMER2
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC2 ) ;
	//* Disable the clock and the interrupts
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC2->TC_IDR = 0xFFFFFFFF;
	//* Enable the clock
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKEN;
	//* Clear status bit
	dummy = AT91C_BASE_TC2->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC2->TC_CMR = AT91C_TC_CLKS_TIMER_DIV4_CLOCK|AT91C_TC_WAVESEL_UP_AUTO|
								AT91C_TC_WAVE; //MCK/(128)  375000 Hz clk to timer2
	//* Config Timer 2 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC2, TIMER2_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Timer2IrqService);
	AT91C_BASE_TC2->TC_IER = AT91C_TC_CPCS;  //  IRQ enable CPC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC2);
	AT91C_BASE_TC2->TC_RC = 3750;		//(375000 / 3750) = 100Hz rate
	AT91C_BASE_TC2->TC_RA = 10;			//
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//start timer

	Timer100Hz = 0;
	SecondTimer = 0;
	MinuteTimer = 0;
	HourTimer = 0;

	//* Setup PWM channel 0,1,2
	//* enable the clock of the PWM system
	//* Configure PIO controllers to periph mode for PWM
	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
		( (unsigned int)RLED | (unsigned int) GLED | (unsigned int) YLED  ) , // Peripheral A
		0); // Peripheral B
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_PWMC ) ;

	*AT91C_PWMC_DIS = AT91C_PWMC_CHID0;
	*AT91C_PWMC_MR = 0;	//turn off DIVA and B
	*AT91C_PWMC_CH0_CMR  = AT91C_PWMC_CPRE_MCK_1024; 	// (PWMC_CH0) MCLK/1024 update duty cycle
	*AT91C_PWMC_CH0_CPRDR = 387; 	// (PWMC_CH0) Channel Period Register 120Hz
	*AT91C_PWMC_CH0_CDTYR = 1; 		// (PWMC_CH0) Channel Duty Cycle Register
	*AT91C_PWMC_CH0_CUPDR  = 1; 	// (PWMC_CH0) Channel Update Register
	*AT91C_PWMC_ENA = AT91C_PWMC_CHID0;

	*AT91C_PWMC_DIS = AT91C_PWMC_CHID1;
	*AT91C_PWMC_CH1_CMR  = AT91C_PWMC_CPRE_MCK_1024; 	// (PWMC_CH1) MCLK/1024 update duty cycle
	*AT91C_PWMC_CH1_CPRDR = 387; 	// (PWMC_CH1) Channel Period Register 120Hz
	*AT91C_PWMC_CH1_CDTYR = 1; 		// (PWMC_CH1) Channel Duty Cycle Register
	*AT91C_PWMC_CH1_CUPDR  = 1; 	// (PWMC_CH1) Channel Update Register
	*AT91C_PWMC_ENA = AT91C_PWMC_CHID1;

	*AT91C_PWMC_DIS = AT91C_PWMC_CHID2;
	*AT91C_PWMC_CH2_CMR  = AT91C_PWMC_CPRE_MCK_1024; 	// (PWMC_CH2) MCLK/1024 update duty cycle
	*AT91C_PWMC_CH2_CPRDR = 387; 	// (PWMC_CH2) Channel Period Register 120Hz
	*AT91C_PWMC_CH2_CDTYR = 1; 		// (PWMC_CH2) Channel Duty Cycle Register
	*AT91C_PWMC_CH2_CUPDR  = 1; 	// (PWMC_CH2) Channel Update Register
	*AT91C_PWMC_ENA = AT91C_PWMC_CHID2;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | S e t G L e d P W M |                               */
/*                      ---------------------                                */
/*    Called to Set the intensity of the GLED. (range 1 to 386)              */
/*    If outside range then LED is full on or off                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetGLedPWM( uint32_t pwm)
{
	if(pwm < LED_MIN)
	{
		*AT91C_PIOA_PER = GLED;
		*AT91C_PIOA_CODR = GLED;
	}
	else if(pwm > LED_MAX)
	{
		*AT91C_PIOA_PER = GLED;
		*AT91C_PIOA_SODR = GLED;
	}
	else
	{
		*AT91C_PIOA_PDR = GLED;
		*AT91C_PWMC_CH2_CUPDR = LED_MAX-pwm;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | S e t Y L e d P W M |                               */
/*                      ---------------------                                */
/*    Called to Set the intensity of the YLED. (range 1 to 386)              */
/*    If outside range then LED is full on or off                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetYLedPWM( uint32_t pwm)
{
	if(pwm < LED_MIN)
	{
		*AT91C_PIOA_PER = YLED;
		*AT91C_PIOA_CODR = YLED;
	}
	else if(pwm > LED_MAX)
	{
		*AT91C_PIOA_PER = YLED;
		*AT91C_PIOA_SODR = YLED;
	}
	else
	{
		*AT91C_PIOA_PDR = YLED;
		*AT91C_PWMC_CH1_CUPDR = LED_MAX-pwm;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | S e t R L e d P W M |                               */
/*                      ---------------------                                */
/*    Called to Set the intensity of the RLED. (range 1 to 386)              */
/*    If outside range then LED is full on or off                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetRLedPWM( uint32_t pwm)
{
	if(pwm < LED_MIN)
	{
		*AT91C_PIOA_PER = RLED;
		*AT91C_PIOA_CODR = RLED;
	}
	else if(pwm > LED_MAX)
	{
		*AT91C_PIOA_PER = RLED;
		*AT91C_PIOA_SODR = RLED;
	}
	else
	{
		*AT91C_PIOA_PDR = RLED;
		*AT91C_PWMC_CH0_CUPDR = LED_MAX-pwm;
	}
}



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------                                      */
/*                     | D e l a y u s |                                     */
/*                      ---------------                                      */
/*    Call to delay without using RTOS timer.                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Delayus(uint32_t delay)
{
volatile uint32_t x __attribute__((unused));
	delay = delay*5;
	while(delay--)
	{
 		x = delay*1234567;
	}
}
