/* ==========================================================================*/
/* - - - - - - - - - - - - -   B o a r d . h   - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Definition File for the WSPR beacon board  HW                          */
/*  Created   2011-09-21   msw                                               */
/*	Modified  2011-09-21   MW                                                */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
//////////////////////////////////////////////////////////////////////////////
#ifndef BOARD_H
#define BOARD_H

#include "AT91SAM7S256.h"
#define __inline static inline
#include "lib_AT91SAM7S256.h"
#define __ramfunc

//#define REV1_PCB 1	//comment out for later revs

#define USE_PRINTF 1


#define AT91C_US_ASYNC_MODE ( AT91C_US_USMODE_NORMAL + AT91C_US_NBSTOP_1_BIT + AT91C_US_PAR_NONE + AT91C_US_CHRL_8_BITS + AT91C_US_CLKS_CLOCK )

/*---------------------------*/
/*  Memories Definition      */
/*---------------------------*/
// The AT91SAM7S256 has a 64-Kbyte SRAM bank, and 256K-Byte Flash

#define  INT_SARM           0x00200000
#define  INT_SARM_REMAP	    0x00000000

#define  INT_FLASH          0x00000000
#define  INT_FLASH_REMAP    0x00100000

#define  BOOTFLASH_START    0x00100000
#define  BOOTFLASH_END    	0x00103FFF

#define  FLASH_PAGE_NB	    1024
#define  FLASH_PAGE_LOCK    64
#define  FLASH_PAGE_SIZE    256

#define FLASH_SECTOR_SIZE (FLASH_PAGE_LOCK*FLASH_PAGE_SIZE)
#define FLASH_SIZE (FLASH_PAGE_NB*FLASH_PAGE_SIZE)

//EEROM type defines
#define USE_64KFRAM
//#define USE_4KEEROM

#define	FRAM_SLAVE_ADR 0x50		//I2C device addresses
#define SI5338_ADR 0x70

#define BOOT_VERSION  ( (uint16_t*)0x00100040) // Location of boot code version number in boot flash memory
#define HW_VERSION  ( (uint16_t*)0x00100042) // Location of HW version number in boot flash memory
#define SERIAL_NUMBER  ( (uint8_t*)0x00100044) // Location of Serial number string in boot flash memory
#define PRODUCT_ID  ( (uint8_t*)0x00100050) // Location of 4 byte Product ID

/*---------------------*/
/* I/O Pin Definitions */
/*---------------------*/
/*----------------------------------------------------------------------------------*/
/*  PORT A I/O Pin Definitions                    AT91SAM7 PIO Multiplexing         */
/*                                      PIO     Flash    Periph A   Periph B   PIN  */
/*----------------------------------------------------------------------------------*/
#define RLED            (1<<0)  /*       PA0 / PGMEN0     PWM0      TIOA0     48    */
#define YLED            (1<<1)  /*       PA1 / PGMEN1     PWM1      TIOB0     47    */
#define GLED            (1<<2)  /*       PA2              PWM2      SCK0      44    */
#define SDA             (1<<3)  /*       PA3              TWD       NPCS3     43    */
#define SCL             (1<<4)  /*       PA4              TWCK      TCLK0     36    */
#define RXD0            (1<<5)  /*       PA5              RXD0      NPCS3     35    */
#define TXD0            (1<<6)  /*       PA6              TXD0      PCK0      34    */
#define PA7             (1<<7)  /*       PA7              RTS0      PWM3      32    */
#define PA8             (1<<8)  /*       PA8              CTS0      ADTRG     31    */
#define DRXD            (1<<9)	/*       PA9              DRXD      NPCS1     30    */
#define DTXD            (1<<10) /*       PA10             DTXD      NPCS2     29    */
#define SW1             (1<<11) /*       PA11             NPCS0     PWM0      28    */
#define SW2             (1<<12) /*       PA12             MISO      PWM1      27    */
#define SW4             (1<<13) /*       PA13             MOSI      PWM2      22    */
#define SW8             (1<<14) /*       PA14             SPCK      PWM3      21    */
#define PPS             (1<<15) /*       PA15             TF        TIOA1     20    */
#define PA16            (1<<16) /*       PA16             TK        TIOB1     19    */
#define GPSON           (1<<17) /*       PA17             TD        PCK1       9    */
#define GPSTOG          (1<<18) /*       PA18             RD        PCK2      10    */
#define PA19            (1<<19) /*       PA19             RK        FIQ       13    */
#define TP2             (1<<20) /*       PA20             RF        IRQ0      16    */
#define GPSTX           (1<<21) /*       PA21             RXD1      PCK1      11    */
#define GPSRX           (1<<22) /*       PA22             TXD1      NPCS3     14    */
#define TP1             (1<<23) /*       PA23             SCK1      PWM0      15    */
#define PA24            (1<<24) /*       PA24             RTS1      PWM1      23    */
#define PA25            (1<<25) /*       PA25             CTS1      PWM2      25    */
#define PA26            (1<<26) /*       PA26             DCD1      TIOA2     26    */
#define PA27            (1<<27) /*       PA27             DTR1      TIOB2     37    */
#define PA28            (1<<28) /*       PA28             DSR1      TCLK1     38    */
#define TXOFF           (1<<29) /*       PA29             RI1       TCLK2     41    */
#define INT5338         (1<<30) /*       PA30             IRQ1      NPCS2     42    */
#define ANTOC           (1<<31) /*       PA31             NPCS1     PCK2      52    */


/*------------------*/
/* USART Definition */
/*------------------*/
#define UART_BAUD	   115200   // Baud rate

#define EXT_OC          25000000   // External oscillator MAINCK
#define MCK             48000000   // MCK = [ (EXT_OC/25) * 96 ] / 2 ( see CStartup_SAM7.c)

/*--------------*/
/* Master Clock */
/*--------------*/
#define MCKKHz          (MCK/1000) //

#endif /* BOARD_H */
