#if !defined( __ASCPDEFS_H__ )
#define __ASCPDEFS_H__
/* ==========================================================================*/
/*  - - - - - - -   P R O T O C O L D E F S . H   - - - - - - - - - - - - -  */
/* ==========================================================================*/
/*    Definition File for the VersaBeacon II protocol and Control Items      */
/*  Created 2010-08-01														 */
/*  Modified 2010-08-01		msw                                              */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
#define VB_INTERFACE_VERSION 1

/*---------------------------------------------------------------------------*/
/*---------------------> Message Header Defines <----------------------------*/
/*---------------------------------------------------------------------------*/
#define LENGTH_MASK 0x1FFF			/* mask for message length               */
#define TYPE_MASK 0xE0				/* mask for upper byte of header         */

#define TYPE_HOST_SET_CITEM (0<<5)
#define TYPE_HOST_REQ_CITEM (1<<5)
#define TYPE_HOST_REQ_CITEM_RANGE (2<<5)
#define TYPE_HOST_DATA_ITEM0 (4<<5)
#define TYPE_HOST_DATA_ITEM1 (5<<5)
#define TYPE_HOST_DATA_ITEM2 (6<<5)
#define TYPE_HOST_DATA_ITEM3 (7<<5)

#define TYPE_TARG_RESP_CITEM (0<<5)
#define TYPE_TARG_UNSOLICITED_CITEM (1<<5)
#define TYPE_TARG_RESP_CITEM_RANGE (2<<5)

#define TYPE_TARG_DATA_ITEM0 (4<<5)
#define TYPE_TARG_DATA_ITEM1 (5<<5)
#define TYPE_TARG_DATA_ITEM2 (6<<5)
#define TYPE_TARG_DATA_ITEM3 (7<<5)

#define TYPE_DATA_ITEM_ACK (3<<5)
 #define DATA_ITEM_ACK_LENGTH (3)

/*  2 byte NAK response to any unimplemented messages */
#define TARG_RESP_NAK (0x0002)

#define MAX_MSG_LENGTH (8192+2)
/*---------------------------------------------------------------------------*/
/*----------------------> Control Item Defines <-----------------------------*/
/*---------------------------------------------------------------------------*/

/*  General Interface Control Items  */
#define CI_GENERAL_INTERFACE_NAME 0x0001
 #define CI_GENERAL_INTERFACE_NAME_REQLEN 4

#define CI_GENERAL_INTERFACE_SERIALNUM 0x0002
 #define CI_GENERAL_INTERFACE_SERIALNUM_REQLEN 4

#define CI_GENERAL_INTERFACE_VERSION 0x0003
 #define CI_GENERAL_INTERFACE_VERSION_REQLEN 4
 #define CI_GENERAL_INTERFACE_VERSION_SETRESPLEN 6

#define CI_GENERAL_HARDFIRM_VERSION 0x0004
 #define CI_GENERAL_HARDFIRM_VERSION_REQLEN 5
 #define CI_GENERAL_HARDFIRM_VERSION_SETRESPLEN 7

#define CI_GENERAL_STATUS_CODE 0x0005
 #define CI_GENERAL_STATUS_CODE_REQLEN 4
 #define CI_GENERAL_STATUS_CODE_SETRESPLEN 5
  #define GENERAL_STATUS_STOPPED 0x00
  #define GENERAL_STATUS_RUNNING 0x01
  #define GENERAL_STATUS_BOOTIDLE 0x0E
  #define GENERAL_STATUS_BOOTBUSY 0x0F
  #define GENERAL_STATUS_BOOTERROR 0x80

#define CI_GENERAL_PRODUCT_ID 0x0009
 #define CI_GENERAL_PRODUCT_ID_REQLEN 4
 #define CI_GENERAL_PRODUCT_ID_SETRESPLEN 8

#define CI_MODULATION 0x028
 #define CI_MODULATION_REQLEN 4
 #define CI_MODULATION_SETRESPLEN 5
   #define CI_MODULATION_CW 0

#define CI_OPMODE 0x02A
 #define CI_OPMODE_REQLEN 4
 #define CI_OPMODE_SETRESPLEN 5
   #define CI_OPMODE_SETUP 0
   #define CI_OPMODE_TUNE 1
   #define CI_OPMODE_WSPR 2
   #define CI_OPMODE_GPSRAW 3
   #define CI_OPMODE_GPS 4
   #define CI_OPMODE_BOOT 5


#define CI_TX_FREQUENCY 0x0120
 #define CI_TX_FREQUENCY_REQLEN 4
 #define CI_TX_FREQUENCY_SETRESPLEN 8

#define CI_TX_RF_POWER 0x0138
 #define CI_TX_RF_POWER_REQLEN 4
 #define CI_TX_RF_POWER_SETRESPLEN 6		//signed 16 bit value in dBm range -16 to +13

#define CI_FREQUENCY_CAL 0x0400
 #define CI_FREQUENCY_CAL_REQLEN 4
 #define CI_FREQUENCY_CAL_SETRESPLEN 6		//signed 16 bit value in dBm range -2000 to +2000
  

#define CI_DEBUG_DATA 0x0500
 #define CI_DEBUG_DATA_REQLEN 4
 #define CI_DEBUG_DATA_SETRESPLEN 83


/*  Software/Firmware Update MODE Control Items  */
#define CI_UPDATE_MODE_CONTROL 0x0300
 #define CI_UPDATE_MODE_CONTROL_SETREQLEN 10
 #define CI_UPDATE_MODE_CONTROL_SETRESPLEN 10
 #define CI_UPDATE_MODE_CONTROL_ENTER 0
 #define CI_UPDATE_MODE_CONTROL_START 1
 #define CI_UPDATE_MODE_CONTROL_END 2
 #define CI_UPDATE_MODE_CONTROL_ABORT 3
 #define CI_UPDATE_MODE_CONTROL_ERASE 4


 #define DEF_PRODID0 0x5A	// The default product id security codes for entering
 #define DEF_PRODID1 0xFF	// the programming mode
 #define DEF_PRODID2 0xA5
 #define DEF_PRODID3 0x00

// Product IDs for Versabeacon II
 #define VB_PRODID0 0x56
 #define VB_PRODID1 0x42
 #define VB_PRODID2 0x32
 #define VB_PRODID3 0x02

#define PRODID_OFFSET 0x50		//offset from start of app code space where product ID is located
#define FLASH_BASE 0x00104000	//base address of application code in flash

#define CI_UPDATE_MODE_PARAMS 0x0302
 #define CI_UPDATE_MODE_PARAMS_SETREQLEN 5
 #define CI_UPDATE_MODE_PARAMS_SETRESPLEN 17

#endif //!defined( __ASCPDEFS_H__ )
